--*****************************************************************************
--*  Copyright (C) 2012 by Michael Fischer
--*
--*  All rights reserved.
--*
--*  Redistribution and use in source and binary forms, with or without 
--*  modification, are permitted provided that the following conditions 
--*  are met:
--*  
--*  1. Redistributions of source code must retain the above copyright 
--*     notice, this list of conditions and the following disclaimer.
--*  2. Redistributions in binary form must reproduce the above copyright
--*     notice, this list of conditions and the following disclaimer in the 
--*     documentation and/or other materials provided with the distribution.
--*  3. Neither the name of the author nor the names of its contributors may 
--*     be used to endorse or promote products derived from this software 
--*     without specific prior written permission.
--*
--*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
--*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
--*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
--*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
--*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
--*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
--*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
--*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
--*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
--*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
--*  SUCH DAMAGE.
--*
--*****************************************************************************
--*  History:
--*
--*  28.07.2012  mifi  First Version, tested with an Altera DE1 board.
--*                    It is inspired by the Terasic Verilog example and
--*                    coded in VHDL.
--*  15.08.2012  mifi  Expanded for 8 segments and read port added.
--*****************************************************************************


--*****************************************************************************
--*  DEFINE: Library                                                          *
--*****************************************************************************

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


--*****************************************************************************
--*  DEFINE: Entity                                                           *
--*****************************************************************************

entity seg7_lut_8_if is
	port (
          -- Avalon Slave bus
		    clk        : in  std_logic := '0';
		    reset      : in  std_logic := '0';
          chipselect : in  std_logic := '0';
		    address    : in  std_logic_vector(2 downto 0) := (others => '0');
          write      : in  std_logic := '0';
          writedata  : in  std_logic_vector(31 downto 0) := (others => '0');
          read       : in  std_logic := '0';
          readdata   : out std_logic_vector(31 downto 0);
      
          -- External bus
		    seg0       : out std_logic_vector(6 downto 0);
		    seg1       : out std_logic_vector(6 downto 0);
		    seg2       : out std_logic_vector(6 downto 0);
		    seg3       : out std_logic_vector(6 downto 0);
		    seg4       : out std_logic_vector(6 downto 0);
		    seg5       : out std_logic_vector(6 downto 0);
		    seg6       : out std_logic_vector(6 downto 0);
		    seg7       : out std_logic_vector(6 downto 0)
	     );
end entity seg7_lut_8_if;

--*****************************************************************************
--*  DEFINE: Architecture                                                     *
--*****************************************************************************

architecture syn of seg7_lut_8_if is

   --
   -- Define all constants here
   --
   
   constant ADDR_DATA : integer := 0; -- Read / Write


   --
   -- Define all components which are included here
   --
   
   component seg7_lut is
      port (
             dig_in     : in  std_logic_vector(3 downto 0);
             seg_out    : out std_logic_vector(6 downto 0)
           );
   end component seg7_lut;


   --
   -- Define all local signals (like static data) here
   --
   
   signal data     : std_logic_vector(31 downto 0) := (others => '0');
   signal read_mux : std_logic_vector(31 downto 0) := (others => '0');  

begin

   ----------------------------------------------
   -- Register File
   ----------------------------------------------

   process(clk)
   begin
      if rising_edge(clk) then
         if (write = '1') then
         
            if (ADDR_DATA = unsigned(address)) then   -- ADDR_DATA
               data <= writedata;
            end if;             
            
         end if;
      end if; 
   end process;


   ----------------------------------------------
   -- Read Multiplexer
   ----------------------------------------------
     
   read_mux <= data when (ADDR_DATA = unsigned(address)) else (others => '0'); 

   process(clk)
   begin
      if rising_edge(clk) then
         if (read = '1') then
            readdata <= read_mux;
         end if;
      end if;
   end process;      


   ----------------------------------------------
   -- External output
   ----------------------------------------------

   inst_seg0 : seg7_lut
      port map ( 
                 dig_in  => data(3 downto 0),
                 seg_out => seg0
               );

   inst_seg1 : seg7_lut
      port map ( 
                 dig_in  => data(7 downto 4),
                 seg_out => seg1
               );

   inst_seg2 : seg7_lut
      port map ( 
                 dig_in  => data(11 downto 8),
                 seg_out => seg2
               );

   inst_seg3 : seg7_lut
      port map ( 
                 dig_in  => data(15 downto 12),
                 seg_out => seg3
               );
               
   inst_seg4 : seg7_lut
      port map ( 
                 dig_in  => data(19 downto 16),
                 seg_out => seg4
               );
               
   inst_seg5 : seg7_lut
      port map ( 
                 dig_in  => data(23 downto 20),
                 seg_out => seg5
               );
               
   inst_seg6 : seg7_lut
      port map ( 
                 dig_in  => data(27 downto 24),
                 seg_out => seg6
               );
               
   inst_seg7 : seg7_lut
      port map ( 
                 dig_in  => data(31 downto 28),
                 seg_out => seg7
               );               

end architecture syn;

-- *** EOF ***
